@@ -17,10 +17,11 @@ specifyModelDA <- function(syntax = NULL,
                            quad.range = Inf,
                            adaptive.quad = FALSE,
                            adaptive.frequency = 3,
-                           impute.na = FALSE,
+                           missing = FALSE,
                            orthogonal.x = FALSE,
                            orthogonal.y = FALSE,
-                           auto.split.syntax = FALSE) {
+                           auto.split.syntax = FALSE,
+                           cluster = NULL) {
   if (!is.null(syntax)) parTable <- modsemify(syntax)
   stopif(is.null(parTable), "No parTable found")
 
@@ -33,7 +34,7 @@ specifyModelDA <- function(syntax = NULL,
     syntax     <- parTableToSyntax(parTable)
     cov.syntax <- parTableToSyntax(parTableCovModel)
   }
-  
+
   checkParTableDA(parTable)
   # additions to lavaan-syntax for optimizer
   lavOptimizerSyntaxAdditions <- ""
@@ -54,7 +55,7 @@ specifyModelDA <- function(syntax = NULL,
   allVarsInInts <- unique(unlist(varsInts))
   xis           <- getXis(parTable, checkAny = TRUE)
   numXis        <- length(xis)
-  
+
   omegaAndSortedXis <- sortXisConstructOmega(xis, varsInts, etas, intTerms,
                                              method = method, double = double)
   xis <- omegaAndSortedXis$sortedXis # get sorted xis according to interaction terms
@@ -67,7 +68,8 @@ specifyModelDA <- function(syntax = NULL,
   numAllIndsXis <- length(allIndsXis)
 
   # clean data
-  data <- cleanAndSortData(data, allIndsXis, allIndsEtas, impute.na = impute.na)
+  data.cleaned <- prepDataModsemDA(data, allIndsXis, allIndsEtas,
+                                   missing = missing, cluster = cluster)
 
   # measurement model x
   listLambdaX <- constructLambda(xis, indsXis, parTable = parTable,
@@ -132,14 +134,14 @@ specifyModelDA <- function(syntax = NULL,
 
   # mean etas
   listAlpha <- constructAlpha(etas, parTable = parTable,
-                              mean.observed = mean.observed) 
-  alpha      <- listAlpha$numeric                            
+                              mean.observed = mean.observed)
+  alpha      <- listAlpha$numeric
   labelAlpha <- listAlpha$label
 
   # mean xis
   listBeta0 <- constructAlpha(xis, parTable = parTable,
                               mean.observed = mean.observed)
-  beta0      <- listBeta0$numeric                   
+  beta0      <- listBeta0$numeric
   labelBeta0 <- listBeta0$label
 
   # quadratic terms
@@ -167,14 +169,20 @@ specifyModelDA <- function(syntax = NULL,
   selectSubSigma2ThetaEpsilon <-
     getSelectSubSigma2ThetaEpsilon(fullSigma2ThetaEpsilon, latentEtas = latentEtas,
                                    method = method)
-  fullU <- constructFullU(fullL2 = fullL2, N = NROW(data), etas = etas, method = method)
+  fullU <- constructFullU(fullL2 = fullL2, N = data.cleaned$n, etas = etas, method = method)
 
   scalingInds <- getScalingInds(indsEtas, R = emptyR, latentEtas = latentEtas,
                                 method = method)
-  selectThetaEpsilon <- selectThetaEpsilon(lambdaY, thetaEpsilon,
-                                           scalingInds, method = method)
-  subThetaEpsilon <- constructSubThetaEpsilon(indsEtas, thetaEpsilon,
-                                              scalingInds, method = method)
+
+  # 1 = truly latent variables, 2 = latent variables with single indicators
+  selectThetaEpsilon1 <- selectThetaEpsilon1(indsEtas, thetaEpsilon,
+                                             scalingInds, method = method)
+  selectThetaEpsilon2 <- selectThetaEpsilon2(indsEtas, thetaEpsilon,
+                                             scalingInds, method = method)
+  subThetaEpsilon1 <- constructSubThetaEpsilon1(indsEtas, thetaEpsilon,
+                                                scalingInds, method = method)
+  subThetaEpsilon2 <- constructSubThetaEpsilon2(indsEtas, thetaEpsilon,
+                                                scalingInds, method = method)
 
   covModel <- covModel(cov.syntax, method = method, parTable = parTableCovModel,
                        xis.main = xis, parTable.main = parTable)
@@ -198,9 +206,10 @@ specifyModelDA <- function(syntax = NULL,
     omegaEtaXi   = omegaEtaXi,
     omegaXiXi    = omegaXiXi,
 
-    selectScalingY     = selectScalingY,
-    selectThetaEpsilon = selectThetaEpsilon,
-    selectBetaRows     = selectBetaRows,
+    selectScalingY      = selectScalingY,
+    selectThetaEpsilon1 = selectThetaEpsilon1,
+    selectThetaEpsilon2 = selectThetaEpsilon2,
+    selectBetaRows      = selectBetaRows,
 
     emptyR = emptyR,
     fullR  = fullR,
@@ -216,7 +225,8 @@ specifyModelDA <- function(syntax = NULL,
     colsR = colnames(emptyR),
     rowsR = rownames(emptyR),
 
-    subThetaEpsilon = subThetaEpsilon)
+    subThetaEpsilon1 = subThetaEpsilon1,
+    subThetaEpsilon2 = subThetaEpsilon2)
 
   labelMatrices <- list(
     lambdaX      = labelLambdaX,
@@ -238,13 +248,13 @@ specifyModelDA <- function(syntax = NULL,
     omegaXiXi  = labelOmegaXiXi)
 
   k <- omegaAndSortedXis$k
-  quad <- quadrature(m, k, quad.range = quad.range, adaptive = adaptive.quad, 
+  quad <- quadrature(m, k, quad.range = quad.range, adaptive = adaptive.quad,
                      adaptive.frequency = adaptive.frequency)
 
   model <- list(
     info = list(
-      N             = NROW(data),
-      ncol          = NCOL(data),
+      N             = data.cleaned$n,
+      ncol          = data.cleaned$k,
       xis           = xis,
       etas          = etas,
       numXis        = numXis,
@@ -259,17 +269,20 @@ specifyModelDA <- function(syntax = NULL,
       kOmegaEta     = getK_NA(omegaEtaXi, labelOmegaEtaXi),
       nonLinearXis  = nonLinearXis,
       mean.observed = mean.observed,
+      has.interaction = NROW(intTerms) > 0L,
       lavOptimizerSyntaxAdditions = lavOptimizerSyntaxAdditions
     ),
 
-    data          = data,
+    data          = data.cleaned,
+    data.raw      = data,
     quad          = quad,
     matrices      = matrices,
     labelMatrices = labelMatrices,
     syntax        = syntax,
     cov.syntax    = cov.syntax,
     parTable      = parTable,
-    covModel      = covModel
+    covModel      = covModel,
+    lavaan.fit    = NULL
   )
 
   model$constrExprs <- getConstrExprs(parTable, model$covModel$parTable)
@@ -281,8 +294,9 @@ specifyModelDA <- function(syntax = NULL,
     model$gradientStruct  <- getGradientStruct(model, theta = model$theta)
   }
 
-  if (checkModel) 
-    preCheckModel(model = model, covModel = covModel, method = method)
+  if (checkModel)
+    preCheckModel(model = model, covModel = covModel, method = method,
+                  missing = missing)
 
   model
 }
@@ -325,16 +339,26 @@ interceptsToParTable <- function(matrixNA, matrixEst, matrixSE, matrixLabel) {
 }
 
 
-omegaToParTable <- function(omegaNA, omegaEst, omegaSE, omegaLabel) {
+omegaToParTable <- function(omegaNA, omegaEst, omegaSE, omegaLabel, parTable.in = NULL) {
   rows <- rownames(omegaEst)
   cols <- colnames(omegaEst)
 
+  C <- \(x, y) sprintf("%s:%s", x, y)
+  getIntTerm <- function(lhs, rhs) {
+    xz <- C(lhs, rhs)
+    zx <- C(rhs, lhs)
+
+    if (is.null(parTable.in))         xz
+    else if (xz %in% parTable.in$rhs) xz
+    else                              zx
+  }
+
   parTable <- NULL
   for (row in rows) for (col in cols) {
     if (!is.na(omegaNA[row, col]) && omegaLabel[row, col] == "") next
     eta     <- getEtaRowLabelOmega(row)
     x       <- getXiRowLabelOmega(row)
-    intTerm <- paste0(x, ":", col)
+    intTerm <- getIntTerm(lhs = x, rhs = col)
 
     newRow <- data.frame(lhs = eta, op = "~", rhs = intTerm,
                          label = omegaLabel[row, col], est = omegaEst[row, col],
@@ -350,6 +374,7 @@ mainModelToParTable <- function(finalModel, method = "lms") {
   matricesSE    <- finalModel$matricesSE
   matricesNA    <- finalModel$matricesNA
   matricesLabel <- finalModel$labelMatrices
+  parTable.in   <- finalModel$parTable
 
   if (is.null(matricesSE)) matricesSE <- matricesNA
 
@@ -395,13 +420,15 @@ mainModelToParTable <- function(finalModel, method = "lms") {
   newRows <- omegaToParTable(matricesNA$omegaXiXi,
                              matricesEst$omegaXiXi,
                              matricesSE$omegaXiXi,
-                             matricesLabel$omegaXiXi)
+                             matricesLabel$omegaXiXi,
+                             parTable.in = parTable.in)
   parTable <- rbind(parTable, newRows)
 
   newRows <- omegaToParTable(matricesNA$omegaEtaXi,
                              matricesEst$omegaEtaXi,
                              matricesSE$omegaEtaXi,
-                             matricesLabel$omegaEtaXi)
+                             matricesLabel$omegaEtaXi,
+                             parTable.in = parTable.in)
   parTable <- rbind(parTable, newRows)
 
   # Intercepts
@@ -502,8 +529,8 @@ customParamsToParTable <- function(model, coefs, se) {
 
 
 modelToParTable <- function(model, coefs = NULL, se = NULL, method = "lms", calc.se = TRUE) {
-  parTable <- rbind(mainModelToParTable(model, method = method),
-                    covModelToParTable(model, method = method))
+  parTable <- rbind(covModelToParTable(model, method = method),
+                    mainModelToParTable(model, method = method))
 
   if (!is.null(coefs) && !is.null(se) && !is.null(names(se))) {
     parTable <- rbind(parTable, customParamsToParTable(model, coefs, se))
@@ -521,9 +548,149 @@ modelToParTable <- function(model, coefs = NULL, se = NULL, method = "lms", calc
 
   if (!calc.se) parTable$std.error <- NA  # when std.errors are not computed, static constraints
                                           # will get Non-NA std.errors, which is incorrect
-                                          # this is naturally corrected for when calculating the 
+                                          # this is naturally corrected for when calculating the
                                           # std.errors, but not when calc.se == FALSE
   parTable[!is.na(parTable$std.error) &
            parTable$std.error == -999, "std.error"] <- NA  # replace -999 with NA
+
+  # Sort parTable before returning
+  sortParTableDA(parTable = parTable, model = model)
+}
+
+
+getConvergenceMessage <- function(converged, iterations) {
+  pattern <- if (isTRUE(converged)) {
+    "\nmodsem (%s) ended normally after %d iterations\n\n"
+  } else {
+    paste0(
+           "modsem (%s) did NOT end normally after %d iterations\n",
+           "** WARNING ** Estimates below are most likely unreliable\n"
+    )
+  }
+  sprintf(pattern, PKG_INFO$version, iterations)
+}
+
+
+finalizeModelEstimatesDA <- function(model,
+                                     theta,
+                                     method = c("lms","qml"),
+                                     data,
+                                     logLik,
+                                     iterations,
+                                     converged,
+                                     optimizer,
+                                     # SE / FIM controls
+                                     calc.se = TRUE,
+                                     FIM = "observed",
+                                     OFIM.hessian = FALSE,
+                                     EFIM.S = 3e4,
+                                     EFIM.parametric = TRUE,
+                                     robust.se = FALSE,
+                                     epsilon = 1e-6,
+                                     cr1s = TRUE,
+                                     R.max = 1e6,
+                                     verbose = FALSE,
+                                     # method-specific extras
+                                     P = NULL,                 # only used by LMS
+                                     includeStartModel = FALSE,
+                                     startModel = NULL) {
+
+  method <- match.arg(method)
+  NA__ <- -999
+
+  # coefficients and filled model
+  lavCoefs <- getLavCoefs(model = model, theta = theta, method = method)
+  # (fillPhi is relevant for LMS, harmless for QML when ignored)
+  finalModel <- fillModel(model, theta, fillPhi = (method == "lms"), method = method)
+
+  # keep NA "skeletons" for printing and SE attachment
+  emptyModel <- getEmptyModel(parTable = model$parTable,
+                              cov.syntax = model$cov.syntax,
+                              parTableCovModel = model$covModel$parTable,
+                              mean.observed = model$info$mean.observed,
+                              method = method)
+  finalModel$matricesNA <- emptyModel$matrices
+  finalModel$covModelNA <- emptyModel$covModel
+
+  # information matrix + SE
+  typeSE <- if (!calc.se) "none" else if (robust.se) "robust" else "standard"
+
+  fim.args <- list(model = model,
+                   finalModel = finalModel,
+                   theta = theta,
+                   data = data,
+                   method = method,
+                   EFIM.S = EFIM.S,
+                   hessian = OFIM.hessian,
+                   calc.se = calc.se,
+                   EFIM.parametric = EFIM.parametric,
+                   verbose = verbose,
+                   FIM = FIM,
+                   robust.se = robust.se,
+                   epsilon = epsilon,
+                   cr1s = cr1s,
+                   R.max = R.max,
+                   NA__ = NA__)
+
+  # only LMS uses the conditional P; QML ignores it
+  if (!is.null(P) && method == "lms") fim.args$P <- P
+
+  FIMo <- do.call(calcFIM_da, fim.args)
+
+  SE <- calcSE_da(calc.se = calc.se,
+                  FIMo$vcov.all,
+                  rawLabels = FIMo$raw.labels,
+                  NA__ = NA__)
+
+  modelSE <- getSE_Model(model, se = SE, method = method,
+                         n.additions = FIMo$n.additions)
+  finalModel$matricesSE <- modelSE$matrices
+  finalModel$covModelSE <- modelSE$covModel
+
+  parTable <- modelToParTable(finalModel,
+                              coefs = lavCoefs$all,
+                              se = SE,
+                              method = method,
+                              calc.se = calc.se)
+  parTable <- addZStatsParTable(parTable)
+
+  out <- list(
+    model            = finalModel,
+    method           = method,
+    optimizer        = optimizer,
+    data             = data,
+    theta            = theta,
+    coefs.all        = lavCoefs$all,
+    coefs.free       = lavCoefs$free,
+    parTable         = modsemParTable(parTable),
+    originalParTable = model$parTable,
+    logLik           = logLik,
+    iterations       = iterations,
+    convergence      = isTRUE(converged),
+    convergence.msg  = getConvergenceMessage(converged, iterations),
+    type.se          = typeSE,
+    type.estimates   = "unstandardized",
+    info.quad        = if (method == "lms") getInfoQuad(model$quad) else NULL,
+    FIM              = FIMo$FIM,
+    vcov.all         = FIMo$vcov.all,
+    vcov.free        = FIMo$vcov.free,
+    information      = FIMo$type
+  )
+
+  if (isTRUE(includeStartModel))
+    out$start.model <- startModel
+
+  out
+}
+
+
+addZStatsParTable <- function(parTable, se.col = "std.error", est.col = "est",
+                              z.col = "z.value", p.col = "p.value",
+                              ci.l = "ci.lower", ci.u = "ci.upper") {
+  parTable[[z.col]] <- parTable[[est.col]] / parTable[[se.col]]
+  parTable[[p.col]] <- 2 * stats::pnorm(-abs(parTable[[z.col]]))
+  parTable[[ci.l]]  <- parTable[[est.col]] - CI_WIDTH * parTable[[se.col]]
+  parTable[[ci.u]]  <- parTable[[est.col]] + CI_WIDTH * parTable[[se.col]]
+
   parTable
 }
