@@ -24,7 +24,7 @@
 #'   separate regression lines. Each distinct value in \code{vals_z} defines a separate
 #'   group (plotted with a different color). If \code{rescale=TRUE}, these values
 #'   are also assumed to be in standardized units.
-#' @param model An object of class \code{\link{modsem_pi}}, \code{\link{modsem_da}}, 
+#' @param model An object of class \code{\link{modsem_pi}}, \code{\link{modsem_da}},
 #'   \code{\link{modsem_mplus}}, or possibly a \code{lavaan} object. Must be a fitted
 #'   SEM model containing paths for \code{y ~ x + z + x:z}.
 #' @param alpha_se A numeric value in \eqn{[0, 1]} specifying the transparency of
@@ -114,11 +114,11 @@
 #'                  model = est2)
 #' }
 plot_interaction <- function(x, z, y, xz = NULL, vals_x = seq(-3, 3, .001),
-                             vals_z, model, alpha_se = 0.15, digits = 2, 
-                             ci_width = 0.95, ci_type = "confidence", rescale = TRUE, 
+                             vals_z, model, alpha_se = 0.15, digits = 2,
+                             ci_width = 0.95, ci_type = "confidence", rescale = TRUE,
                              standardized = FALSE, ...) {
-  slopes <- simple_slopes(x = x, z = z, y = y, model = model, vals_x = vals_x, vals_z = vals_z, 
-                          rescale = rescale, ci_width = ci_width, ci_type = ci_type, 
+  slopes <- simple_slopes(x = x, z = z, y = y, model = model, vals_x = vals_x, vals_z = vals_z,
+                          rescale = rescale, ci_width = ci_width, ci_type = ci_type,
                           standardized = standardized, ...)
   df <- as.data.frame(slopes)
   df$cat_z <- as.factor(round(df$vals_z, digits))
@@ -136,8 +136,8 @@ plot_interaction <- function(x, z, y, xz = NULL, vals_x = seq(-3, 3, .001),
     ggplot2::geom_smooth(method = "lm", formula = "y ~ x", se = FALSE) +
     ggplot2::geom_ribbon(ggplot2::aes(ymin = ci.lower, ymax = ci.upper, fill = cat_z),
                          alpha = alpha_se, linewidth = 0, linetype = "blank") +
-    ggplot2::labs(x = x, y = y, colour = z, fill = z) + 
-    ggplot2::ggtitle(sprintf("Marginal Effects of %s on %s, Given %s", x, y, z)) + 
+    ggplot2::labs(x = x, y = y, colour = z, fill = z) +
+    ggplot2::ggtitle(sprintf("Marginal Effects of %s on %s, Given %s", x, y, z)) +
     ggplot2::theme_bw()
 }
 
@@ -156,10 +156,11 @@ plot_interaction <- function(x, z, y, xz = NULL, vals_x = seq(-3, 3, .001),
 #' @param sig.level The alpha-criterion for the confidence intervals (default is 0.05).
 #' @param alpha alpha setting used in \code{ggplot} (i.e., the opposite of opacity)
 #' @param detail The number of generated data points to use for the plot (default is 1000). You can increase this value for smoother plots.
-#' @param sd.line A thick black line showing \code{+/- sd.line * sd(z)}. NOTE: This line will be truncated by \code{min_z} and \code{max_z} if 
+#' @param sd.line A thick black line showing \code{+/- sd.line * sd(z)}. NOTE: This line will be truncated by \code{min_z} and \code{max_z} if
 #' the sd.line falls outside of \code{[min_z, max_z]}.
 #' @param standardized Should coefficients be standardized beforehand?
 #' @param ... Additional arguments (currently not used).
+#'
 #' @return A \code{ggplot} object showing the interaction plot with regions of significance.
 #' @details
 #' The function calculates the simple slopes of the predictor variable \code{x} on the outcome variable \code{y} at different levels of the moderator variable \code{z}. It uses the Johnson-Neyman technique to identify the regions of \code{z} where the effect of \code{x} on \code{y} is statistically significant.
@@ -179,33 +180,31 @@ plot_interaction <- function(x, z, y, xz = NULL, vals_x = seq(-3, 3, .001),
 #' \dontrun{
 #' library(modsem)
 #'
-#' m1 <-  ' 
-#'   visual  =~ x1 + x2 + x3 
+#' m1 <-  '
+#'   visual  =~ x1 + x2 + x3
 #'   textual =~ x4 + x5 + x6
 #'   speed   =~ x7 + x8 + x9
-#' 
+#'
 #'   visual ~ speed + textual + speed:textual
 #' '
-#' 
+#'
 #' est <- modsem(m1, data = lavaan::HolzingerSwineford1939, method = "ca")
 #' plot_jn(x = "speed", z = "textual", y = "visual", model = est, max_z = 6)
 #' }
-#' @importFrom ggplot2 ggplot aes geom_line geom_ribbon geom_vline annotate scale_fill_manual labs theme_minimal
 #' @export
-plot_jn <- function(x, z, y, xz = NULL, model, min_z = -3, max_z = 3, 
-                    sig.level = 0.05, alpha = 0.2, detail = 1000, 
+plot_jn <- function(x, z, y, xz = NULL, model, min_z = -3, max_z = 3,
+                    sig.level = 0.05, alpha = 0.2, detail = 1000,
                     sd.line = 2, standardized = FALSE, ...) {
-  # Check if model is a valid object
+
   stopif(!inherits(model, c("modsem_da", "modsem_mplus", "modsem_pi", "lavaan")),
          "model must be of class 'modsem_pi', 'modsem_da', 'modsem_mplus', or 'lavaan'")
 
-  # If interaction term is not specified, create it
   if (is.null(xz)) xz <- paste(x, z, sep = ":")
   xz <- c(xz, reverseIntTerm(xz))
   if (!inherits(model, c("modsem_da", "modsem_mplus")) && !inherits(model, "lavaan")) {
     xz <- stringr::str_remove_all(xz, ":")
   }
-    
+
   if (inherits(model, "lavaan")) {
     vcov <- lavaan::vcov
     coef <- lavaan::coef
@@ -214,129 +213,98 @@ plot_jn <- function(x, z, y, xz = NULL, model, min_z = -3, max_z = 3,
 
   if (standardized) {
     parTable <- standardized_estimates(model, correction = TRUE)
-  } else parTable <- parameter_estimates(model)
+  } else {
+    parTable <- parameter_estimates(model)
+  }
   parTable <- getMissingLabels(parTable)
 
-  # mean and sd x
+  # z mean/sd and plotting window
   mean_z <- getMean(z, parTable = parTable)
   sd_z   <- sqrt(calcCovParTable(x = z, y = z, parTable = parTable))
   min_z  <- min_z + mean_z
   max_z  <- max_z + mean_z
 
-  # Extract coefficients
-  beta_x <- parTable[parTable$lhs == y & parTable$rhs == x & parTable$op == "~", "est"]
+  # coefficients
+  beta_x  <- parTable[parTable$lhs == y & parTable$rhs == x   & parTable$op == "~", "est"]
   beta_xz <- parTable[parTable$lhs == y & parTable$rhs %in% xz & parTable$op == "~", "est"]
-
-  stopif(length(beta_x) == 0, "Coefficient for x not found in model")
+  stopif(length(beta_x)  == 0, "Coefficient for x not found in model")
   stopif(length(beta_xz) == 0, "Coefficient for interaction term not found in model")
 
-  # Extract variance-covariance matrix
   VCOV <- vcov(model)
-
-  label_beta_x  <- parTable[parTable$lhs == y & parTable$rhs == x & parTable$op == "~", "label"]
+  label_beta_x  <- parTable[parTable$lhs == y & parTable$rhs == x   & parTable$op == "~", "label"]
   label_beta_xz <- parTable[parTable$lhs == y & parTable$rhs %in% xz & parTable$op == "~", "label"]
 
-  # Extract variances and covariances
-  var_beta_x         <- VCOV[label_beta_x, label_beta_x]
+  var_beta_x         <- VCOV[label_beta_x,  label_beta_x]
   var_beta_xz        <- VCOV[label_beta_xz, label_beta_xz]
-  cov_beta_x_beta_xz <- VCOV[label_beta_x, label_beta_xz]
+  cov_beta_x_beta_xz <- VCOV[label_beta_x,  label_beta_xz]
 
   nobs <- nobs(model)
   npar <- length(coef(model))
-
   df_resid <- nobs - npar
-  
   if (df_resid < 1) {
     warning2("Degrees of freedom for residuals must be greater than 0. ",
-             "The model may have fewer observations than parameters.\n",
              "Using sample size instead of degrees of freedom")
     df_resid <- nobs
   }
 
-  # Critical t-value
   t_crit <- stats::qt(1 - sig.level / 2, df_resid)
 
-  # Quadratic equation components
+  # Johnson–Neyman roots
   A <- beta_xz^2 - t_crit^2 * var_beta_xz
   B <- 2 * beta_x * beta_xz - 2 * t_crit^2 * cov_beta_x_beta_xz
   C <- beta_x^2 - t_crit^2 * var_beta_x
+  disc <- B^2 - 4 * A * C
 
-  discriminant <- B^2 - 4 * A * C
-
+  significant_everywhere <- FALSE
   if (A == 0) {
-    # Linear case
     if (B != 0) {
-      z_jn <- -C / B
-      significant_everywhere <- FALSE
-      z_lower <- z_jn
-      z_upper <- z_jn
+      z_jn <- -C / B; z_lower <- z_jn; z_upper <- z_jn
     } else {
       message("No regions where the effect transitions between significant and non-significant.")
       significant_everywhere <- TRUE
     }
-  } else if (discriminant < 0) {
+  } else if (disc < 0) {
     message("No regions where the effect transitions between significant and non-significant.")
     significant_everywhere <- TRUE
-
-  } else if (discriminant == 0) {
-    # One real root
-    z_jn <- -B / (2 * A)
-    significant_everywhere <- FALSE
-    z_lower <- z_jn
-    z_upper <- z_jn
+  } else if (disc == 0) {
+    z_jn <- -B / (2 * A); z_lower <- z_jn; z_upper <- z_jn
   } else {
-    # Two real roots
-    z1 <- (-B + sqrt(discriminant)) / (2 * A)
-    z2 <- (-B - sqrt(discriminant)) / (2 * A)
-    z_lower <- min(z1, z2)
-    z_upper <- max(z1, z2)
-    significant_everywhere <- FALSE
+    z1 <- (-B + sqrt(disc)) / (2 * A)
+    z2 <- (-B - sqrt(disc)) / (2 * A)
+    z_lower <- min(z1, z2); z_upper <- max(z1, z2)
   }
 
-  z_range <- seq(min_z, max_z, length.out = detail)
+  # grid and simple slopes
+  z_range  <- seq(min_z, max_z, length.out = detail)
+  slope    <- beta_x + beta_xz * z_range
+  SE_slope <- sqrt(var_beta_x + z_range^2 * var_beta_xz + 2 * z_range * cov_beta_x_beta_xz)
+  t_value  <- slope / SE_slope
+  p_value  <- 2 * (1 - stats::pt(abs(t_value), df_resid))
+  significant <- p_value < sig.level
 
-  # if not defined here (as opposed to only in the dataframe) we will get 
-  # some bogus notes in the R CMD check
-  slope         <- beta_x + beta_xz * z_range
-  SE_slope      <- sqrt(var_beta_x + z_range ^ 2 * var_beta_xz + 2 * z_range * cov_beta_x_beta_xz)
-  t_value       <- slope / SE_slope
-  p_value       <- 2 * (1 - stats::pt(abs(t_value), df_resid))
-  significant   <- p_value < sig.level
-  lower_all     <- slope - t_crit * SE_slope
-  upper_all     <- slope + t_crit * SE_slope
-  lower_sig     <- ifelse(significant,  lower_all, NA)
-  upper_sig     <- ifelse(significant,  upper_all, NA)
-  lower_sig     <- ifelse(significant, lower_all, NA)
-  upper_sig     <- ifelse(significant, upper_all, NA)
-  lower_nsig    <- ifelse(!significant, lower_all, NA)
-  upper_nsig    <- ifelse(!significant, upper_all, NA)
-  significance  <- ifelse(significant, sprintf("p < %s", sig.level), "n.s.")
-  line_segments <- cumsum(c(0, diff(as.numeric(significant)) != 0))
+  lower_all <- slope - t_crit * SE_slope
+  upper_all <- slope + t_crit * SE_slope
+
+  siglabel <- sprintf("p < %s", sig.level)
+  significance_chr <- ifelse(significant, siglabel, "n.s.")
+  Significance <- factor(significance_chr, levels = c(siglabel, "n.s."))
+
+  # split into contiguous runs to avoid polygon bleed
+  run_id <- cumsum(c(0, diff(as.integer(significant)) != 0))
 
-  # Create the data frame
   df_plot <- data.frame(
-      z            = z_range,
-      slope        = slope,
-      SE           = SE_slope,
-      t            = t_value,
-      p            = p_value,
-      significant  = significant,
-      upper_all    = upper_all,
-      lower_all    = lower_all,
-      upper_sig    = upper_sig,
-      lower_sig    = lower_sig,
-      upper_nsig   = upper_nsig,
-      lower_nsig   = lower_nsig,
-      Significance = significance,
-      # Really f***ing ugly, but works... (if not the line will sometimes be 
-      # on color...)
-      line_segments = line_segments
+    z = z_range,
+    slope = as.numeric(slope),
+    lower_all = as.numeric(lower_all),
+    upper_all = as.numeric(upper_all),
+    significant = significant,
+    Significance = Significance,
+    run_id = run_id
   )
-  
-  # get info for thick line segment
+
+  # SD-band segment
   x_start <- mean_z - sd.line * sd_z
   x_end   <- mean_z + sd.line * sd_z
-
   if (x_start < min_z && x_end > max_z) {
     warning2("Truncating SD-range on the right and left!")
   } else if (x_start < min_z) {
@@ -344,59 +312,76 @@ plot_jn <- function(x, z, y, xz = NULL, model, min_z = -3, max_z = 3,
   } else if (x_end > max_z) {
     warning2("Truncating SD-range on the right!")
   }
-
-  x_start     <- max(x_start, min_z)
-  x_end       <- min(x_end, max_z)
-  y_start     <- y_end <- 0
+  x_start <- max(x_start, min_z)
+  x_end   <- min(x_end, max_z)
+  y_start <- y_end <- 0
   hline_label <- sprintf("+/- %s SDs of %s", sd.line, z)
+  data_hline <- data.frame(x_start, x_end, y_start, y_end, hline_label)
 
-  data_hline <- data.frame(x_start = x_start, x_end = x_end, 
-                           y_start = y_start, y_end = y_end,
-                           hline_label = hline_label)
+  # unified legend for colour/fill
+  breaks <- c(siglabel, "n.s.", hline_label)
+  values <- structure(c("cyan3", "red", "black"), names = breaks)
 
-  siglabel <- sprintf("p < %s", sig.level)
-  breaks   <- c(siglabel, "n.s.", hline_label)
-  values   <- structure(c("cyan3", "red", "black"), names = breaks)
+  y_range <- range(c(df_plot$lower_all, df_plot$upper_all, 0), na.rm = TRUE)
+  if (!all(is.finite(y_range))) y_range <- c(-1, 1)
 
+  # plot
   p <- ggplot2::ggplot(df_plot, ggplot2::aes(x = z, y = slope)) +
-    ggplot2::geom_line(ggplot2::aes(color = significance, group = line_segments), linewidth = 1) +
-    ggplot2::geom_ribbon(ggplot2::aes(ymin = lower_nsig, ymax = upper_nsig, fill = "n.s."), alpha = alpha) +
-    ggplot2::geom_ribbon(ggplot2::aes(ymin = lower_sig, ymax = upper_sig, fill = sprintf("p < %s", sig.level)), alpha = alpha) +
-    ggplot2::labs(x = z, y = paste("Slope of", x, "on", y)) +
-    ggplot2::geom_hline(yintercept = 0, color="black", linewidth = 0.5) + 
-    suppressWarnings(ggplot2::geom_segment(
-        mapping = aes(x = x_start, xend = x_end, y = y_start, yend = y_end, 
-                      color = hline_label, fill = hline_label), 
-        data = data_hline, linewidth = 1.5)
-    ) + 
+    # single ribbon, split into runs; fill follows Significance
+    ggplot2::geom_ribbon(
+      ggplot2::aes(ymin = lower_all, ymax = upper_all,
+                   fill = Significance, group = run_id),
+      alpha = alpha, na.rm = TRUE
+    ) +
+    # line coloured by Significance, also split into runs for color change
+    ggplot2::geom_line(
+      ggplot2::aes(color = Significance, group = run_id),
+      linewidth = 1, na.rm = TRUE
+    ) +
+    ggplot2::geom_hline(yintercept = 0, color = "black", linewidth = 0.5) +
+    suppressWarnings(
+      ggplot2::geom_segment(
+        mapping = ggplot2::aes(x = x_start, xend = x_end, y = y_start, yend = y_end,
+                               color = hline_label, fill = hline_label),
+        data = data_hline, linewidth = 1.5
+      )
+    ) +
     ggplot2::ggtitle("Johnson-Neyman Plot") +
-    ggplot2::scale_fill_manual(name = "", values = values, breaks = breaks) +
-    ggplot2::scale_color_manual(name = "", values = values, breaks = breaks) +
-    ggplot2::guides(fill = ggplot2::guide_legend(title = ""),
-                    color = ggplot2::guide_legend(title = "")) +
+    ggplot2::scale_discrete_manual(
+      aesthetics = c("colour", "fill"),
+      name = "",
+      values = values,
+      breaks = breaks,
+      drop = FALSE
+    ) +
+    ggplot2::scale_y_continuous(limits = y_range) +
+    ggplot2::labs(x = z, y = paste("Slope of", x, "on", y)) +
     ggplot2::theme_minimal()
 
+  # only show JN lines if there is a transition in the plotted window
+  has_transition_in_window <- any(diff(as.integer(df_plot$significant)) != 0, na.rm = TRUE)
+  if (!significant_everywhere && has_transition_in_window) {
+    top_y <- suppressWarnings(max(df_plot$slope[is.finite(df_plot$slope)], na.rm = TRUE))
+    if (!is.finite(top_y)) top_y <- y_range[2]
 
-  if (!significant_everywhere) {
     if (exists("z_jn")) {
-      # Single JN point
-      if (z_jn >= min_z && z_jn <= max_z) {
+      if (is.finite(z_jn) && z_jn >= min_z && z_jn <= max_z) {
         p <- p + ggplot2::geom_vline(xintercept = z_jn, linetype = "dashed", color = "red") +
-          ggplot2::annotate("text", x = z_jn, y = max(df_plot$slope), label = paste("JN point:", round(z_jn, 2)),
+          ggplot2::annotate("text", x = z_jn, y = top_y,
+                            label = paste("JN point:", round(z_jn, 2)),
                             hjust = -0.1, vjust = 1, color = "black")
       }
     } else {
-      # Two JN points
-      if (z_lower >= min_z && z_lower <= max_z) {
-        p <- p +
-          ggplot2::geom_vline(xintercept = z_lower, linetype = "dashed", color = "red") +
-          ggplot2::annotate("text", x = z_lower, y = max(df_plot$slope), label = paste("JN point:", round(z_lower, 2)),
+      if (is.finite(z_lower) && z_lower >= min_z && z_lower <= max_z) {
+        p <- p + ggplot2::geom_vline(xintercept = z_lower, linetype = "dashed", color = "red") +
+          ggplot2::annotate("text", x = z_lower, y = top_y,
+                            label = paste("JN point:", round(z_lower, 2)),
                             hjust = -0.1, vjust = 1, color = "black")
       }
-      if (z_upper >= min_z && z_upper <= max_z) {
-        p <- p +
-          ggplot2::geom_vline(xintercept = z_upper, linetype = "dashed", color = "red") +
-          ggplot2::annotate("text", x = z_upper, y = max(df_plot$slope), label = paste("JN point:", round(z_upper, 2)),
+      if (is.finite(z_upper) && z_upper >= min_z && z_upper <= max_z) {
+        p <- p + ggplot2::geom_vline(xintercept = z_upper, linetype = "dashed", color = "red") +
+          ggplot2::annotate("text", x = z_upper, y = top_y,
+                            label = paste("JN point:", round(z_upper, 2)),
                             hjust = -0.1, vjust = 1, color = "black")
       }
     }
@@ -466,22 +451,22 @@ plot_jn <- function(x, z, y, xz = NULL, model, min_z = -3, max_z = 3,
 #'   PBC =~ pbc1 + pbc2 + pbc3
 #'   INT =~ int1 + int2 + int3
 #'   BEH =~ b1 + b2
-#' 
+#'
 #' # Inner Model (Based on Steinmetz et al., 2011)
 #'   INT ~ ATT + SN + PBC
 #'   BEH ~ INT + PBC
 #'   BEH ~ PBC:INT
 #' "
-#' 
+#'
 #' est2 <- modsem(tpb, TPB, method = "lms", nodes = 32)
 #' plot_surface(x = "INT", z = "PBC", y = "BEH", model = est2)
 #' }
 #'
 #' @export
-plot_surface <- function(x, z, y, xz = NULL, model, 
-                         min_x = -3, max_x = 3, 
+plot_surface <- function(x, z, y, xz = NULL, model,
+                         min_x = -3, max_x = 3,
                          min_z = -3, max_z = 3,
-                         standardized = FALSE, 
+                         standardized = FALSE,
                          detail = 1e-2, ...) {
   stopif(!isModsemObject(model) && !isLavaanObject(model), "model must be of class ",
          "'modsem_pi', 'modsem_da', 'modsem_mplus' or 'lavaan'")
