@@ -7,11 +7,11 @@ transformedSolutionCOEFS <- function(object,
                                      center = TRUE,
                                      standardize = TRUE,
                                      ...) {
-  stopif(!inherits(object, c("modsem_da", "modsem_pi", "lavaan")),
-         "The model must be of class `modsem_da`, `modsem_pi` or `lavaan`!")
+  stopif(!inherits(object, c("modsem_da", "modsem_pi", "lavaan", "modsem_mplus")),
+         "The model must be of class `modsem_da`, `modsem_mplus`, `modsem_pi` or `lavaan`!")
 
   isLav <- inherits(object, "lavaan")
-  isDA  <- inherits(object, "modsem_da")
+  isDA  <- inherits(object, c("modsem_da", "modsem_mplus"))
 
   if (isLav) {
     vcov <- lavaan::vcov # load vcov and coef from lavaan if dealing with a lavaan object
@@ -39,7 +39,7 @@ transformedSolutionCOEFS <- function(object,
            "variances when centering the model!\n", immediate. = FALSE)
 
     if (isDA) parTable <- meanInteractions(parTable) # get means for interaction terms
-    parTable <- var_interactions(parTable, ignore.means = TRUE)
+    parTable <- var_interactions(parTable, ignore.means = TRUE, mc.reps = mc.reps)
   }
 
   lVs      <- getLVs(parTable)
@@ -110,7 +110,7 @@ transformedSolutionCOEFS <- function(object,
   # Center interactions
   if (center) {
     COEFS <- centerInteractionsCOEFS(parTable, COEFS = COEFS) # re-estimate path-coefficients
-    parTable <- parTable[parTable$op != "~1", ]
+    parTable <- parTable[!parTable$op %in% c("~1", "|"), ]
   }
 
   # Unstandardized copies
@@ -212,8 +212,7 @@ transformedSolutionCOEFS <- function(object,
                                      COEFS.std = COEFS,
                                      COEFS.ustd = COEFS.ustd,
                                      variances = variances,
-                                     intTerms = intTerms
-    )
+                                     intTerms = intTerms)
 
   }
   # recalculate custom parameters
@@ -313,11 +312,9 @@ correctStdSolutionCOEFS <- function(parTable,
                                     variances,
                                     intTerms) {
   for (XZ in intTerms) {
-    elems <- stringr::str_split_fixed(XZ, ":", 2)
-    X     <- elems[[1]]
-    Z     <- elems[[2]]
+    elems <- stringr::str_split(XZ, pattern = ":")[[1L]]
 
-    vars  <- as.data.frame(variances[c(X, Z)])
+    vars  <- as.data.frame(variances[elems])
     sds   <- sqrt(vars)
 
     rowsXZ <- parTable[parTable$rhs == XZ & parTable$op == "~", , drop = FALSE]
@@ -508,8 +505,8 @@ centerInteractionsCOEFS <- function(parTable, COEFS, center.means = TRUE,
 
   if (center.means) {
     innerVars <- unique(unlist(parTable[parTable$op == "~", c("rhs", "lhs")]))
-    interceptLabels <- parTable[parTable$lhs %in% innerVars & 
-                                parTable$op == "~1", label.col] 
+    interceptLabels <- parTable[parTable$lhs %in% innerVars &
+                                parTable$op == "~1", label.col]
 
     for (label in interceptLabels)
       COEFS[[label]] <- 0
@@ -519,9 +516,9 @@ centerInteractionsCOEFS <- function(parTable, COEFS, center.means = TRUE,
 }
 
 
-addTransformedEstimatesPT <- function(parTable, 
-                                      FUN, 
-                                      pass.parTable = TRUE, 
+addTransformedEstimatesPT <- function(parTable,
+                                      FUN,
+                                      pass.parTable = TRUE,
                                       values.to = "transformed",
                                       values.from = "est",
                                       merge.by = c("lhs", "op", "rhs"),
@@ -532,14 +529,14 @@ addTransformedEstimatesPT <- function(parTable,
     parTable.transform[[values.to]] <- parTable.transform[[values.from]]
     parTable.transform <- parTable.transform[c(merge.by, values.to)]
 
-    leftJoin(left   = parTable, 
+    leftJoin(left   = parTable,
              right  = parTable.transform,
              by     = merge.by)
 }
-  
 
-applyTransformationByGrouping <- function(parTable, 
-                                          FUN, 
+
+applyTransformationByGrouping <- function(parTable,
+                                          FUN,
                                           groupingcols = c("block", "group"),
                                           ...) {
   if (any(groupingcols %in% colnames(parTable))) {
@@ -549,11 +546,11 @@ applyTransformationByGrouping <- function(parTable,
     parTable.out <- NULL
 
     for (i in seq_len(NROW(categories))) {
-      grouping       <- structure(unlist(categories[i, ]), 
+      grouping       <- structure(unlist(categories[i, ]),
                                   names = groupingcols)
       parTable.out_i <- FUN(..., grouping = grouping)
 
-      if (is.null(parTable.out_i)) 
+      if (is.null(parTable.out_i))
         next
 
       for (group in names(grouping))
