@@ -3,7 +3,7 @@ m1 <- '
 # Outer Model
   X =~ x1 + x2
   Z =~ z1 + z2
-  Y =~ y1 + y2 
+  Y =~ y1 + y2
 
 # Inner model
   Y ~ X + Z + X:Z
@@ -15,8 +15,10 @@ run <- tryCatch({
   error = function(e) FALSE
 )
 if (run) {
-  mplus <- modsem(m1, oneInt, method = "mplus")
+  mplus <- modsem(m1, oneInt, method = "mplus", estimator = "MLR")
+  standardized_estimates(mplus, type = "modsem")
   print(summary(mplus))
+  print(vcov(mplus))
   plot_interaction(x = "X", z = "Z", y = "Y", xz = "X:Z", vals_z = c(-0.5, 0.5), model = mplus)
 }
 
@@ -34,18 +36,25 @@ tpb <- "
   BEH ~ INT + PBC + INT:PBC
 "
 
-if (run) mplus_tpb <- modsem(tpb, data = TPB, method = "mplus", rcs = TRUE)
-
+if (run) {
+  mplus_tpb <- modsem(tpb, data = TPB, method = "mplus", rcs = TRUE)
+  print(standardized_estimates(mplus_tpb, type = "modsem"))
+  print(standardized_estimates(mplus_tpb, type = "stdyx"))
+}
 
 tpb <- "
 # Outer Model (Based on Hagger et al., 2007)
-  SUBJECTIVE_NORMS =~ sn1 + sn2 
+  SUBJECTIVE_NORMS =~ sn1 + sn2
   PBC =~ pbc1 + pbc2 + pbc3
   INT =~ int1 + int2 + int3
   BEH =~ b1 + b2
 
 # Inner Model (Based on Steinmetz et al., 2011)
-  BEH ~ INT + PBC + INT:PBC:SUBJECTIVE_NORMS + INT:PBC
+  BEH ~ INT + PBC + INT:PBC:SUBJECTIVE_NORMS + INT:PBC + SUBJECTIVE_NORMS
 "
 
-if (run) mplus_tpb_3way <- modsem(tpb, data = TPB, method = "mplus", rcs = TRUE)
+if (run) {
+  mplus_tpb_3way <- modsem(tpb, data = TPB[1:250, ], method = "mplus", rcs = TRUE,
+                           integration = 8)
+  standardized_estimates(mplus_tpb_3way)
+}
