@@ -7,14 +7,17 @@ Z =~ z1 + z2 + z3
 Y ~ X + Z + X:Z
 '
 
-testthat::expect_warning(
-  est <- lavaan::sam(m1, lapplyDf(oneInt, function(x) x - mean(x))),
-  regex = "*switching to naive*"
-)
+sam <- \(...) testthat::expect_warning(lavaan::sam(...),
+                                       regex = "*switching to naive*")
 
-summary(est)
+est <- sam(m1, oneInt)
+parameter_estimates(est)
+
+wrap <- \(expr) (testthat::expect_warning(expr, regex = "Replacing.*"))
+wrap(centered_estimates(est))
+wrap(standardized_estimates(est))
 
 plot_interaction(x = "X", z = "Z", y = "Y", xz = "X:Z",
                  vals_z = c(-0.5, 0.5), model = est)
 plot_jn(x = "X", z = "Z", y = "Y", model = est)
-plot_surface(x = "X", z = "Z", y = "Y", model = est)
\ No newline at end of file
+plot_surface(x = "X", z = "Z", y = "Y", model = est)
